@@ -264,8 +264,14 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 	unsigned int i, n_ptr = 0, n_thumb = 0;
 
 	/* If we are to save IFD 0, we need some extra entries. */
-	if (ifd == data->ifd0)
-		n_ptr = 3;
+	if (ifd == data->ifd0) {
+		if (data->ifd_exif->count)
+			n_ptr++;
+		if (data->ifd_gps->count)
+			n_ptr++;
+		if (data->ifd_interoperability->count)
+			n_ptr++;
+	}
 
 	/*
 	 * If we are to save IFD 1, we can point to the thumbnail if it
@@ -278,7 +284,7 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 	 * Allocate enough memory for all entries
 	 * and the number of entries.
 	 */
-	*ds += (2 + (ifd->count + n_ptr + n_thumb) * 12 + (n_ptr ? 4 : 0));
+	*ds += (2 + (ifd->count + n_ptr + n_thumb) * 12 + 4);
 	*d = realloc (*d, sizeof (char) * *ds);
 
 	/* Save the number of entries */
@@ -296,7 +302,7 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 	offset += 12 * ifd->count;
 
 	/* Save special entries */
-	if (n_ptr) {
+	if (ifd == data->ifd0 && data->ifd_exif->count) {
 
 		/* EXIF_TAG_EXIF_IFD_POINTER */
 		exif_set_short (*d + 6 + offset + 0, data->priv->order,
@@ -309,6 +315,9 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		exif_data_save_data_content (data, data->ifd_exif, d, ds,
 					     *ds - 6);
 		offset += 12;
+	}
+
+	if (ifd == data->ifd0 && data->ifd_gps->count) {
 
 		/* EXIF_TAG_GPS_INFO_IFD_POINTER */
 		exif_set_short (*d + 6 + offset + 0, data->priv->order,
@@ -321,6 +330,9 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		exif_data_save_data_content (data, data->ifd_gps, d, ds,
 					     *ds - 6);
 		offset += 12;
+	}
+
+	if (ifd == data->ifd0 && data->ifd_interoperability->count) {
 
 		/* EXIF_TAG_INTEROPERABILITY_IFD_POINTER */
 		exif_set_short (*d + 6 + offset + 0, data->priv->order,
@@ -333,14 +345,6 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		exif_data_save_data_content (data, data->ifd_interoperability,
 					     d, ds, *ds - 6);
 		offset += 12;
-
-		/*
-		 * We are saving IFD 0. Tell where IFD 1 starts and save
-		 * IFD 1.
-		 */
-		exif_set_long (*d + 6 + offset, data->priv->order, *ds - 6);
-		exif_data_save_data_content (data, data->ifd1, d, ds,
-					     *ds - 6);
 	}
 
 	if (n_thumb) {
@@ -366,7 +370,19 @@ exif_data_save_data_content (ExifData *data, ExifContent *ifd,
 		*ds += data->size;
 		*d = realloc (*d, sizeof (char) * *ds);
 		memcpy (*d + *ds - data->size, data->data, data->size);
+		offset += 12;
 	}
+
+	if (ifd == data->ifd0 && data->ifd1->count) {
+		/*
+		 * We are saving IFD 0. Tell where IFD 1 starts and save
+		 * IFD 1.
+		 */
+		exif_set_long (*d + 6 + offset, data->priv->order, *ds - 6);
+		exif_data_save_data_content (data, data->ifd1, d, ds,
+					     *ds - 6);
+	} else
+		exif_set_long (*d + 6 + offset, data->priv->order, 0);
 }
 
 void
@@ -392,8 +408,8 @@ exif_data_load_data (ExifData *data, const unsigned char *d, unsigned int size)
 	if (size < 6) {
 #ifdef DEBUG
 		printf ("Size too small.\n");
-		return;
 #endif
+		return;
 	}
 	if (!memcmp (d, ExifHeader, 6)) {
 #ifdef DEBUG
@@ -434,7 +450,9 @@ exif_data_load_data (ExifData *data, const unsigned char *d, unsigned int size)
 				break;
 
 			/* Unknown marker or data. Give up. */
+#ifdef DEBUG
 			printf ("EXIF marker not found.\n");
+#endif
 			return;
 		}
 		d++;
@@ -460,14 +478,14 @@ exif_data_load_data (ExifData *data, const unsigned char *d, unsigned int size)
 	if (size < 6) {
 #ifdef DEBUG
 		printf ("Size too small.\n");
-		return;
 #endif
+		return;
 	}
 	if (memcmp (d, ExifHeader, 6)) {
 #ifdef DEBUG
 		printf ("EXIF header not found.\n");
-		return;
 #endif
+		return;
 	}
 
 #ifdef DEBUG
@@ -565,15 +583,41 @@ exif_data_new_from_file (const char *path)
 	unsigned int size;
 	unsigned char *data;
 	ExifData *edata;
+	int marker, ll, lh;
 
-	f = fopen (path, "r");
+	f = fopen (path, "rb");
 	if (!f)
 		return (NULL);
 
-	/* For now, we read the data into memory. Patches welcome... */
-	fseek (f, 0, SEEK_END);
-	size = ftell (f);
-	fseek (f, 0, SEEK_SET);
+	while (1) {
+		while ((marker = fgetc (f)) == 0xff);
+
+		/* JPEG_MARKER_SOI */
+		if (marker == JPEG_MARKER_SOI)
+			continue;
+
+		/* JPEG_MARKER_APP0 */
+		if (marker == JPEG_MARKER_APP0) {
+			lh = fgetc (f);
+			ll = fgetc (f);
+			size = (lh << 8) | ll;
+			if (fseek (f, size - 2, SEEK_CUR) < 0)
+				return (NULL);
+			continue;
+		}
+
+		/* JPEG_MARKER_APP1 */
+		if (marker == JPEG_MARKER_APP1)
+			break;
+
+		/* Unknown marker or data. Give up. */
+		return (NULL);
+	}
+
+	/* EXIF data found. Allocate the necessary memory and read the data. */
+	lh = fgetc (f);
+	ll = fgetc (f);
+	size = (lh << 8) | ll;
 	data = malloc (sizeof (char) * size);
 	if (!data)
 		return (NULL);
@@ -678,6 +722,9 @@ void
 exif_data_foreach_content (ExifData *data, ExifDataForeachContentFunc func,
 			   void *user_data)
 {
+	if (!data || !func)
+		return;
+
 	func (data->ifd0,                 user_data);
 	func (data->ifd1,                 user_data);
 	func (data->ifd_exif,             user_data);
