@@ -26,7 +26,25 @@
 #include <time.h>
 #include <math.h>
 
-#include "exif-i18n.h"
+#ifdef ENABLE_NLS
+#  include <libintl.h>
+#  undef _
+#  define _(String) dgettext (PACKAGE, String)
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  define textdomain(String) (String)
+#  define gettext(String) (String)
+#  define dgettext(Domain,Message) (Message)
+#  define dcgettext(Domain,Message,Type) (Message)
+#  define bindtextdomain(Domain,Directory) (Domain)
+#  define _(String) (String)
+#  define N_(String) (String)
+#endif
+
 #include "exif-utils.h"
 
 //#define DEBUG
@@ -142,7 +160,7 @@ exif_entry_get_value (ExifEntry *e)
 {
 	unsigned int i;
 	ExifByte v_byte;
-	ExifShort v_short, v_short2;
+	ExifShort v_short, v_short2, v_short3, v_short4;
 	ExifLong v_long;
 	ExifSLong v_slong;
 	ExifRational v_rat;
@@ -150,6 +168,10 @@ exif_entry_get_value (ExifEntry *e)
 	static char v[1024], b[1024];
 	const char *c;
 	ExifByteOrder o;
+	double d;
+	ExifEntry *entry;
+
+	bindtextdomain (PACKAGE, LIBEXIF_LOCALEDIR);
 
 	/* We need the byte order */
 	if (!e || !e->parent || !e->parent->parent)
@@ -157,6 +179,8 @@ exif_entry_get_value (ExifEntry *e)
 	o = exif_data_get_byte_order (e->parent->parent);
 
 	memset (v, 0, sizeof (v));
+	memset (b, 0, sizeof (b));
+
 	switch (e->tag) {
 	case EXIF_TAG_EXIF_VERSION:
 		CF (e->format, EXIF_FORMAT_UNDEFINED, v);
@@ -165,6 +189,8 @@ exif_entry_get_value (ExifEntry *e)
 			strncpy (v, "Exif Version 2.0", sizeof (v));
 		else if (!memcmp (e->data, "0210", 4))
 			strncpy (v, "Exif Version 2.1", sizeof (v));
+		else if (!memcmp (e->data, "0220", 4))
+			strncpy (v, "Exif Version 2.2", sizeof (v));
 		else
 			strncpy (v, "Unknown Exif Version", sizeof (v));
 		break;
@@ -190,20 +216,88 @@ exif_entry_get_value (ExifEntry *e)
 			strncat (v, "[None]", sizeof (v));
 		strncat (v, " (Editor)", sizeof (v));
 		break;
+	case EXIF_TAG_FNUMBER:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+		snprintf (v, sizeof (v), "f/%.01f", (float) v_rat.numerator /
+						    (float) v_rat.denominator);
+		break;
 	case EXIF_TAG_APERTURE_VALUE:
 		CF (e->format, EXIF_FORMAT_RATIONAL, v);
 		CC (e->components, 1, v);
 		v_rat = exif_get_rational (e->data, o);
-		snprintf (b, sizeof (b), "%i/%i", (int) v_rat.numerator,
-						  (int) v_rat.denominator);
-		snprintf (v, sizeof (v), "%s (APEX: f%.01f)", b,
+		if (!v_rat.denominator)
+			return (NULL);
+		snprintf (v, sizeof (v), "f/%.01f",
 			  pow (2 , ((float) v_rat.numerator /
 				    (float) v_rat.denominator) / 2.));
 		break;
+	case EXIF_TAG_FOCAL_LENGTH:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+
+		/*
+		 * For calculation of the 35mm equivalent,
+		 * Minolta cameras need a multiplier that depends on the
+		 * camera model.
+		 */
+		d = 0.;
+		entry = exif_content_get_entry (e->parent->parent->ifd0,
+						EXIF_TAG_MAKE);
+		if (entry && entry->data &&
+		    !strncmp (entry->data, "Minolta", 7)) {
+			entry = exif_content_get_entry (e->parent->parent->ifd0,
+							EXIF_TAG_MODEL);
+			if (entry && entry->data) {
+				if (!strncmp (entry->data, "DiMAGE 7", 8))
+					d = 3.9;
+				else if (!strncmp (entry->data, "DiMAGE 5", 8))
+					d = 4.9;
+			}
+		}
+		if (d)
+			snprintf (b, sizeof (b), _(" (35 equivalent: %d mm)"),
+				  (int) (d * (double) v_rat.numerator /
+				  	     (double) v_rat.denominator));
+
+		snprintf (v, sizeof (v), "%.1f mm%s",
+			  (float) v_rat.numerator / (float) v_rat.denominator,
+			  b);
+		break;
+	case EXIF_TAG_SUBJECT_DISTANCE:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+		snprintf (v, sizeof (v), "%.1f m", (float) v_rat.numerator /
+						   (float) v_rat.denominator);
+		break;
+	case EXIF_TAG_EXPOSURE_TIME:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+		d = (double) v_rat.numerator / (double) v_rat.denominator;
+		if (d < 1)
+			snprintf (v, sizeof (v), _("1/%d sec."),
+				  (int) (1. / d));
+		else
+			snprintf (v, sizeof (v), _("%d sec."), (int) d);
+		break;
 	case EXIF_TAG_SHUTTER_SPEED_VALUE:
 		CF (e->format, EXIF_FORMAT_SRATIONAL, v);
 		CC (e->components, 1, v);
 		v_srat = exif_get_srational (e->data, o);
+		if (!v_srat.denominator)
+			return (NULL);
 		snprintf (b, sizeof (b), "%.0f/%.0f sec.",
 			  (float) v_srat.numerator, (float) v_srat.denominator);
 		snprintf (v, sizeof (v), "%s (APEX: %i)", b,
@@ -377,7 +471,32 @@ exif_entry_get_value (ExifEntry *e)
 			strncpy (v, _("Fluorescent"), sizeof (v));
 			break;
 		case 3:
-			strncpy (v, _("Tungsten"), sizeof (v));
+			strncpy (v, _("Tungsten (incandescent light)"),
+				 sizeof (v));
+			break;
+		case 4:
+			strncpy (v, _("Flash"), sizeof (v));
+			break;
+		case 9:
+			strncpy (v, _("Fine weather"), sizeof (v));
+			break;
+		case 10:
+			strncpy (v, _("Cloudy weather"), sizeof (v));
+			break;
+		case 11:
+			strncpy (v, _("Shade"), sizeof (v));
+			break;
+		case 12:
+			strncpy (v, _("Daylight fluorescent"), sizeof (v));
+			break;
+		case 13:
+			strncpy (v, _("Day white fluorescent"), sizeof (v));
+			break;
+		case 14:
+			strncpy (v, _("Cool white fluorescent"), sizeof (v));
+			break;
+		case 15:
+			strncpy (v, _("White fluorescent"), sizeof (v));
 			break;
 		case 17:
 			strncpy (v, _("Standard light A"), sizeof (v));
@@ -397,6 +516,9 @@ exif_entry_get_value (ExifEntry *e)
 		case 22:
 			strncpy (v, _("D75"), sizeof (v));
 			break;
+		case 24:
+			strncpy (v, _("ISO studio tungsten"), sizeof (v));
+			break;
 		case 255:
 			strncpy (v, _("Other"), sizeof (v));
 			break;
@@ -465,6 +587,17 @@ exif_entry_get_value (ExifEntry *e)
 			break;
 		}
 		break;
+	case EXIF_TAG_EXPOSURE_BIAS_VALUE:
+		CF (e->format, EXIF_FORMAT_SRATIONAL, v);
+		CC (e->components, 1, v);
+		v_srat = exif_get_srational (e->data, o);
+		if (!v_srat.denominator)
+			return (NULL);
+		snprintf (v, sizeof (v), "%s%.01f",
+			  v_srat.denominator * v_srat.numerator > 0 ? "+" : "",
+			  (double) v_srat.numerator /
+			  (double) v_srat.denominator);
+		break;
 	case EXIF_TAG_ORIENTATION:
 		CF (e->format, EXIF_FORMAT_SHORT, v);
 		CC (e->components, 1, v);
@@ -573,10 +706,82 @@ exif_entry_get_value (ExifEntry *e)
 			strncpy (v, _("Flash fired."), sizeof (v));
 			break;
 		case 0x0005:
-			strncpy (v, _("Strobe return light not detected."), sizeof (v));
+			strncpy (v, _("Strobe return light not detected."),
+				 sizeof (v));
 			break;
 		case 0x0007:
-			strncpy (v, _("Strobe return light detected."), sizeof (v));
+			strncpy (v, _("Strobe return light detected."),
+				 sizeof (v));
+			break;
+		case 0x000d:
+			strncpy (v, _("Flash fired, compulsory flash mode, "
+				"return light not detected."), sizeof (v));
+			break;
+		case 0x000f:
+			strncpy (v, _("Flash fired, compulsory flash mode, "
+				"return light detected."), sizeof (v));
+			break;
+		case 0x0010:
+			strncpy (v, _("Flash did not fire, compulsory flash "
+				"mode."), sizeof (v));
+			break;
+		case 0x0018:
+			strncpy (v, _("Flash did not fire, auto mode."),
+				 sizeof (v));
+			break;
+		case 0x0019:
+			strncpy (v, _("Flash fired, auto mode."), sizeof (v));
+			break;
+		case 0x001d:
+			strncpy (v, _("Flash fired, auto mode, return light "
+				"not detected."), sizeof (v));
+			break;
+		case 0x001f:
+			strncpy (v, _("Flash fired, auto mode, return light "
+				"detected."), sizeof (v));
+			break;
+		case 0x0020:
+			strncpy (v, _("No flash function."), sizeof (v));
+			break;
+		case 0x0041:
+			strncpy (v, _("Flash fired, red-eye reduction mode."),
+				 sizeof (v));
+			break;
+		case 0x0045:
+			strncpy (v, _("Flash fired, red-eye reduction mode, "
+				"return light not detected."), sizeof (v));
+			break;
+		case 0x0047:
+			strncpy (v, _("Flash fired, red-eye reduction mode, "
+				"return light detected."), sizeof (v));
+			break;
+		case 0x0049:
+			strncpy (v, _("Flash fired, compulsory flash mode, "
+				"red-eye reduction mode."), sizeof (v));
+			break;
+		case 0x004d:
+			strncpy (v, _("Flash fired, compulsory flash mode, "
+				"red-eye reduction mode, return light not "
+				"detected."), sizeof (v));
+			break;
+		case 0x004f:
+			strncpy (v, _("Flash fired, compulsory flash mode, "
+				"red-eye reduction mode, return light "
+				"detected."), sizeof (v));
+			break;
+		case 0x0059:
+			strncpy (v, _("Flash fired, auto mode, red-eye "
+				"reduction mode."), sizeof (v));
+			break;
+		case 0x005d:
+			strncpy (v, _("Flash fired, auto mode, return light "
+				"not detected, red-eye reduction mode."),
+				sizeof (v));
+			break;
+		case 0x005f:
+			strncpy (v, _("Flash fired, auto mode, return light "
+				"detected, red-eye reduction mode."),
+				sizeof (v));
 			break;
 		default:
 			snprintf (v, sizeof (v), "%i", v_short);
@@ -588,6 +793,197 @@ exif_entry_get_value (ExifEntry *e)
 		snprintf (v, sizeof (v), _("%i bytes unknown data"),
 			  (int) e->components);
 		break;
+	case EXIF_TAG_CUSTOM_RENDERED:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Normal process"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Custom process"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_EXPOSURE_MODE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Auto exposure"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Manual exposure"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("Auto bracket"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_WHITE_BALANCE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Auto white balance"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Manual white balance"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_SCENE_CAPTURE_TYPE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Standard"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Landscape"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("Portrait"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("Night scene"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_GAIN_CONTROL:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Normal"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Low gain up"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("High gain up"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("Low gain down"), sizeof (v));
+			break;
+		case 4:
+			strncpy (v, _("High gain down"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_SATURATION:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Normal"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Low saturation"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("High saturation"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_CONTRAST:
+	case EXIF_TAG_SHARPNESS:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Normal"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Soft"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("Hard"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_SUBJECT_DISTANCE_RANGE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("Unknown"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("Macro"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("Close view"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("Distant view"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_SUBJECT_AREA:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		switch (e->components) {
+		case 2:
+			v_short  = exif_get_short (e->data, o);
+			v_short2 = exif_get_short (e->data + 2, o);
+			snprintf (v, sizeof (v), "(x,y) = (%i,%i)",
+				  v_short, v_short2);
+			break;
+		case 3:
+			v_short  = exif_get_short (e->data, o);
+			v_short2 = exif_get_short (e->data + 2, o);
+			v_short3 = exif_get_short (e->data + 4, o);
+			snprintf (v, sizeof (v), _("Within distance %i of "
+				"(x,y) = (%i,%i)"), v_short3, v_short,
+				v_short2);
+			break;
+		case 4:
+			v_short  = exif_get_short (e->data, o);
+			v_short2 = exif_get_short (e->data + 2, o);
+			v_short3 = exif_get_short (e->data + 4, o);
+			v_short4 = exif_get_short (e->data + 6, o);
+			snprintf (v, sizeof (v), _("Within rectangle "
+				"(width %i, height %i) around "
+				"(x,y) = (%i,%i)"), v_short3, v_short4,
+				v_short, v_short2);
+			break;
+		default:
+			snprintf (v, sizeof (v), _("Unexpected number "
+				"of components (%li, expected 2, 3, or 4)."),
+				e->components);
+		}
 	default:
 		switch (e->format) {
 		case EXIF_FORMAT_UNDEFINED:
@@ -677,6 +1073,208 @@ exif_entry_get_value (ExifEntry *e)
 	return (v);
 }
 
+const char *
+exif_entry_get_value_brief (ExifEntry *e)
+{
+	ExifByteOrder o;
+	ExifEntry *entry;
+	static char v[1024];
+	ExifRational v_rat;
+	ExifSRational v_srat;
+	ExifShort v_short;
+	double d;
+
+	/* We need the byte order */
+	if (!e || !e->parent || !e->parent->parent)
+		return (NULL);
+	o = exif_data_get_byte_order (e->parent->parent);
+
+	memset (v, 0, sizeof (v));
+
+	switch (e->tag) {
+	case EXIF_TAG_FOCAL_LENGTH:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+		
+		/*
+		 * For calculation of the 35mm equivalent,
+		 * Minolta cameras need a multiplier that depends on the
+		 * camera model.
+		 */
+		d = (double) v_rat.numerator / (double) v_rat.denominator;
+		entry = exif_content_get_entry (e->parent->parent->ifd0,
+					        EXIF_TAG_MAKE);
+		if (entry && entry->data &&
+		    !strncmp (entry->data, "Minolta", 7)) {
+			entry = exif_content_get_entry (e->parent->parent->ifd0,
+							EXIF_TAG_MODEL);
+			if (entry && entry->data) {
+				if (!strncmp (entry->data, "DiMAGE 7", 8))
+					d *= 3.9;
+				else if (!strncmp (entry->data, "DiMAGE 5", 8))
+					d *= 4.9;
+			}
+		}
+		snprintf (v, sizeof (v), "%dmm",
+				(int) (d * (double) v_rat.numerator /
+				           (double) v_rat.denominator));
+		break;
+	case EXIF_TAG_EXPOSURE_TIME:
+		CF (e->format, EXIF_FORMAT_RATIONAL, v);
+		CC (e->components, 1, v);
+		v_rat = exif_get_rational (e->data, o);
+		if (!v_rat.denominator)
+			return (NULL);
+		d = (double) v_rat.numerator / (double) v_rat.denominator;
+		if (d < 1.)
+			snprintf (v, sizeof (v), "1/%d", (int) (1. / d));
+		else
+			snprintf (v, sizeof (v), "%d", (int) d);
+		break;
+	case EXIF_TAG_SHUTTER_SPEED_VALUE:
+		CF (e->format, EXIF_FORMAT_SRATIONAL, v);
+		CC (e->components, 1, v);
+		v_srat = exif_get_srational (e->data, o);
+		if (!v_srat.denominator)
+			return (NULL);
+		d = 1. / pow (2., (double) v_srat.numerator /
+				  (double) v_srat.denominator);
+		if (d < 1.)
+			snprintf (v, sizeof (v), "1/%d", (int) (1. / d));
+		else
+			snprintf (v, sizeof (v), "%d", (int) d);
+		break;
+	case EXIF_TAG_METERING_MODE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("unknown"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("average"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("center-weight"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("spot"), sizeof (v));
+			break;
+		case 4:
+			strncpy (v, _("multi-spot"), sizeof (v));
+			break;
+		case 5:
+			strncpy (v, _("matrix"), sizeof (v));
+			break;
+		case 6:
+			strncpy (v, _("partial"), sizeof (v));
+			break;
+		case 255:
+			strncpy (v, _("other"), sizeof (v));
+			break;
+		default:
+			snprintf (v, sizeof (v), "%i", v_short);
+			break;
+		}
+		break;
+	case EXIF_TAG_LIGHT_SOURCE:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0:
+			strncpy (v, _("sunny"), sizeof (v));
+			break;
+		case 1:
+			strncpy (v, _("fluorescent"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("tungsten"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("cloudy"), sizeof (v));
+			break;
+		default:
+			return (exif_entry_get_value (e));
+		}
+		break;
+	case EXIF_TAG_RESOLUTION_UNIT:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 2:
+			strncpy (v, _("in"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("cm"), sizeof (v));
+			break;
+		default:
+			return (NULL);
+		}
+		break;
+	case EXIF_TAG_EXPOSURE_PROGRAM:
+		CF (e->format, EXIF_FORMAT_SHORT, v);
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 1:
+			strncpy (v, _("manual"), sizeof (v));
+			break;
+		case 2:
+			strncpy (v, _("normal"), sizeof (v));
+			break;
+		case 3:
+			strncpy (v, _("aperture"), sizeof (v));
+			break;
+		case 4:
+			strncpy (v, _("shutter"), sizeof (v));
+			break;
+		case 5:
+			strncpy (v, _("creative"), sizeof (v));
+			break;
+		case 6:
+			strncpy (v, _("action"), sizeof (v));
+			break;
+		case 7:
+			strncpy (v, _("portrait"), sizeof (v));
+			break;
+		case 8:
+			strncpy (v, _("landscape"), sizeof (v));
+			break;
+		default:
+			return (exif_entry_get_value (e));
+		}
+		break;
+	case EXIF_TAG_FLASH:
+		CF (e->format, EXIF_FORMAT_SHORT, v); 
+		CC (e->components, 1, v);
+		v_short = exif_get_short (e->data, o);
+		switch (v_short) {
+		case 0x0001:
+			strncpy (v, _("yes"), sizeof (v));
+			break;
+		case 0x0005:
+			strncpy (v, _("w/o strobe"), sizeof (v));
+			break;
+		case 0x0007:
+			strncpy (v, _("w. strobe"), sizeof (v));
+			break;
+		default:
+			return (exif_entry_get_value (e));
+		}
+		break;
+	default:
+		return (exif_entry_get_value (e));
+	}
+
+	return (v);
+}
+
 void
 exif_entry_initialize (ExifEntry *e, ExifTag tag)
 {
@@ -711,6 +1309,11 @@ exif_entry_initialize (ExifEntry *e, ExifTag tag)
 	case EXIF_TAG_SENSING_METHOD:
 	case EXIF_TAG_PHOTOMETRIC_INTERPRETATION:
 	case EXIF_TAG_COMPRESSION:
+	case EXIF_TAG_EXPOSURE_MODE:
+	case EXIF_TAG_WHITE_BALANCE:
+	case EXIF_TAG_FOCAL_LENGTH_IN_35MM_FILM:
+	case EXIF_TAG_GAIN_CONTROL:
+	case EXIF_TAG_SUBJECT_DISTANCE_RANGE:
 
 	/* SHORT, 1 component, default 0 */
 	case EXIF_TAG_IMAGE_WIDTH:
@@ -718,6 +1321,11 @@ exif_entry_initialize (ExifEntry *e, ExifTag tag)
 	case EXIF_TAG_EXPOSURE_PROGRAM:
 	case EXIF_TAG_LIGHT_SOURCE:
 	case EXIF_TAG_METERING_MODE:
+	case EXIF_TAG_CUSTOM_RENDERED:
+	case EXIF_TAG_SCENE_CAPTURE_TYPE:
+	case EXIF_TAG_CONTRAST:
+	case EXIF_TAG_SATURATION:
+	case EXIF_TAG_SHARPNESS:
 		e->components = 1;
 		e->format = EXIF_FORMAT_SHORT;
 		e->size = exif_format_get_size (e->format) * e->components;
@@ -803,6 +1411,7 @@ exif_entry_initialize (ExifEntry *e, ExifTag tag)
 	case EXIF_TAG_APERTURE_VALUE:
 	case EXIF_TAG_COMPRESSED_BITS_PER_PIXEL:
 	case EXIF_TAG_PRIMARY_CHROMATICITIES:
+	case EXIF_TAG_DIGITAL_ZOOM_RATIO:
 		e->components = 1;
 		e->format = EXIF_FORMAT_RATIONAL;
 		e->size = exif_format_get_size (e->format) * e->components;
